@@ -2,6 +2,9 @@
 #include <cstddef>
 #include <iostream>
 #include <memory>
+#ifndef PRECICE_NO_MPI
+#include <mpi.h>
+#endif
 #include <stdexcept>
 #include <string>
 #include <string_view>
@@ -55,6 +58,40 @@ try {
   std::abort();
 }
 
+void precicef_create_with_communicator_(
+    const char *participantName,
+    const char *configFileName,
+    const int  *solverProcessIndex,
+    const int  *solverProcessSize,
+    const int  *communicator,
+    int         participantNameLength,
+    int         configFileNameLength)
+try {
+#ifndef PRECICE_NO_MPI
+  // MPI_Comm_f2c requires an MPI_Fint argument, which may differ from int.
+  // However, the Fortran standard guarantees interoperability of the int type,
+  // so that is what is passed. Do the conversion in case int is not identical
+  // to MPI_Fint.
+  MPI_Fint f_communicator = static_cast<MPI_Fint>(*communicator);
+  auto     c_communicator = MPI_Comm_f2c(f_communicator);
+  impl                    = std::make_unique<precice::Participant>(
+      precice::impl::strippedStringView(participantName, participantNameLength),
+      precice::impl::strippedStringView(configFileName, configFileNameLength),
+      *solverProcessIndex,
+      *solverProcessSize,
+      &c_communicator);
+#else
+  PRECICE_WARN("preCICE was configured without MPI but you passed an MPI communicator. preCICE ignores the communicator and continues.");
+  impl = std::make_unique<precice::Participant>(
+      precice::impl::strippedStringView(participantName, participantNameLength),
+      precice::impl::strippedStringView(configFileName, configFileNameLength),
+      *solverProcessIndex,
+      *solverProcessSize);
+#endif
+} catch (::precice::Error &e) {
+  std::abort();
+}
+
 void precicef_initialize_()
 try {
   PRECICE_CHECK(impl != nullptr, errormsg);
@@ -506,15 +543,15 @@ try {
 
 void precicef_get_mesh_vertex_ids_and_coordinates_(
     const char *meshName,
-    const int   size,
+    const int  *size,
     int        *ids,
     double     *coordinates,
     int         meshNameLength)
 try {
   PRECICE_CHECK(impl != nullptr, errormsg);
   auto sv              = precice::impl::strippedStringView(meshName, meshNameLength);
-  auto coordinatesSize = static_cast<unsigned long>(impl->getMeshDimensions(sv) * size);
-  impl->getMeshVertexIDsAndCoordinates(sv, {ids, static_cast<unsigned long>(size)}, {coordinates, coordinatesSize});
+  auto coordinatesSize = static_cast<unsigned long>(impl->getMeshDimensions(sv) * *size);
+  impl->getMeshVertexIDsAndCoordinates(sv, {ids, static_cast<unsigned long>(*size)}, {coordinates, coordinatesSize});
 } catch (::precice::Error &e) {
   std::abort();
 }
