@@ -24,13 +24,12 @@ void ConstantPreconditioner::initialize(std::vector<size_t> &svs)
 
   PRECICE_ASSERT(_factors.size() == _subVectorSizes.size());
 
-  int offset = 0;
   for (size_t k = 0; k < _subVectorSizes.size(); k++) {
+    auto offset = _subVectorOffsets[k];
     for (size_t i = 0; i < _subVectorSizes[k]; i++) {
       _weights[i + offset]    = 1.0 / _factors[k];
       _invWeights[i + offset] = _factors[k];
     }
-    offset += _subVectorSizes[k];
   }
 }
 
