@@ -8,8 +8,7 @@
 #include <string>
 #include "logging/Logger.hpp"
 
-namespace precice {
-namespace com {
+namespace precice::com {
 
 /// This Queue is intended for SocketCommunication to push requests which should be sent onto it.
 /// It ensures that the invocations of asio::aSend are done serially.
@@ -20,11 +19,11 @@ public:
   SocketSendQueue() = default;
   ~SocketSendQueue();
 
-  SocketSendQueue(SocketSendQueue const &) = delete;
+  SocketSendQueue(SocketSendQueue const &)            = delete;
   SocketSendQueue &operator=(SocketSendQueue const &) = delete;
 
   /// Put data in the queue, start processing the queue.
-  void dispatch(std::shared_ptr<Socket> sock, boost::asio::const_buffers_1 data, std::function<void()> callback);
+  void dispatch(std::shared_ptr<Socket> sock, boost::asio::const_buffer data, std::function<void()> callback);
 
   /// Notifies the queue that the last asynchronous send operation has completed.
   void sendCompleted();
@@ -34,18 +33,17 @@ private:
   void process();
 
   struct SendItem {
-    std::shared_ptr<Socket>      sock;
-    boost::asio::const_buffers_1 data;
-    std::function<void()>        callback;
+    std::shared_ptr<Socket>   sock;
+    boost::asio::const_buffer data;
+    std::function<void()>     callback;
   };
 
   /// The queue, containing items to asynchronously send using boost.asio.
   std::deque<SendItem> _itemQueue;
   /// The mutex protecting access to the queue
-  std::mutex _queueMutex;
+  std::mutex _queueMutex{};
   /// Is the queue allowed to start another asynchronous send?
   bool _ready = true;
 };
 
-} // namespace com
-} // namespace precice
+} // namespace precice::com
