@@ -1,9 +1,9 @@
 #pragma once
 
+#include <map>
 #include <memory>
 
-namespace precice {
-namespace cplscheme {
+namespace precice::cplscheme {
 
 class CouplingScheme;
 class CouplingSchemeConfiguration;
@@ -12,6 +12,6 @@ class CouplingData;
 using PtrCouplingScheme              = std::shared_ptr<CouplingScheme>;
 using PtrCouplingSchemeConfiguration = std::shared_ptr<CouplingSchemeConfiguration>;
 using PtrCouplingData                = std::shared_ptr<CouplingData>;
+using DataMap                        = std::map<int, PtrCouplingData>; /// Map that links DataID to CouplingData
 
-} // namespace cplscheme
-} // namespace precice
+} // namespace precice::cplscheme
