@@ -2,9 +2,9 @@
 
 #include <memory>
 #include <string>
+#include <string_view>
 
-namespace precice {
-namespace logging {
+namespace precice::logging {
 
 /// Struct used to capture the original location of a log request
 struct LogLocation {
@@ -17,9 +17,9 @@ struct LogLocation {
 class Logger {
 public:
   /** Creates a logger for a given module.
-   * @param[in] the name of the module 
+   * @param[in] the name of the module
    */
-  explicit Logger(std::string module);
+  explicit Logger(std::string_view module);
 
   Logger(const Logger &other);
   Logger(Logger &&other) noexcept;
@@ -30,11 +30,11 @@ public:
 
   ///@name Logging operations
   ///@{
-  void error(LogLocation loc, const std::string &mess) noexcept;
-  void warning(LogLocation loc, const std::string &mess) noexcept;
-  void info(LogLocation loc, const std::string &mess) noexcept;
-  void debug(LogLocation loc, const std::string &mess) noexcept;
-  void trace(LogLocation loc, const std::string &mess) noexcept;
+  void error(LogLocation loc, std::string_view mess) noexcept;
+  void warning(LogLocation loc, std::string_view mess) noexcept;
+  void info(LogLocation loc, std::string_view mess) noexcept;
+  void debug(LogLocation loc, std::string_view mess) noexcept;
+  void trace(LogLocation loc, std::string_view mess) noexcept;
   ///@}
 
 private:
@@ -45,8 +45,15 @@ private:
   std::unique_ptr<LoggerImpl> _impl;
 };
 
-} // namespace logging
-} // namespace precice
+/// Utility function to log an error and throw an exception of given type
+template <class Error>
+inline void logErrorAndThrow [[noreturn]] (precice::logging::Logger &log, precice::logging::LogLocation location, const std::string &message)
+{
+  log.error(location, message);
+  throw Error{message};
+}
+
+} // namespace precice::logging
 
 // Include LogMacros here, because using it works only together with a Logger
 #include "LogMacros.hpp"
