@@ -27,8 +27,7 @@ MeshConfiguration::MeshConfiguration(
       _meshDimensionsMap(),
       _dataConfig(std::move(config)),
       _meshes(),
-      _neededMeshes(),
-      _meshIdManager(new utils::ManageUniqueIDs())
+      _neededMeshes()
 {
   using namespace xml;
   std::string doc;
@@ -68,8 +67,7 @@ void MeshConfiguration::xmlTagCallback(
     int         dimensions = tag.getIntAttributeValue(ATTR_DIMENSIONS);
     insertMeshToMeshDimensionsMap(name, dimensions);
     PRECICE_ASSERT(dimensions != 0);
-    PRECICE_ASSERT(_meshIdManager);
-    _meshes.push_back(std::make_shared<Mesh>(name, dimensions, _meshIdManager->getFreeID()));
+    _meshes.push_back(std::make_shared<Mesh>(name, dimensions, _meshIdManager.getFreeID()));
   } else if (tag.getName() == TAG_DATA) {
     std::string name  = tag.getStringAttributeValue(ATTR_NAME);
     bool        found = false;
