@@ -1,11 +1,11 @@
 #include <Eigen/Core>
-#include <Eigen/src/Core/Matrix.h>
 #include <algorithm>
 #include <deque>
 #include <iosfwd>
 #include <memory>
 #include <string>
 #include <vector>
+
 #include "logging/Logger.hpp"
 #include "mesh/BoundingBox.hpp"
 #include "mesh/Data.hpp"
@@ -29,9 +29,10 @@ BOOST_AUTO_TEST_SUITE(MeshTests)
 
 BOOST_AUTO_TEST_SUITE(MeshTests)
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(BoundingBoxCOG_2D)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Eigen::Vector2d coords0(2, 0);
   Eigen::Vector2d coords1(-1, 4);
   Eigen::Vector2d coords2(0, 1);
@@ -60,9 +61,10 @@ BOOST_AUTO_TEST_CASE(BoundingBoxCOG_2D)
   }
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(BoundingBoxCOG_3D)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Eigen::Vector3d coords0(2, 0, -3);
   Eigen::Vector3d coords1(-1, 4, 8);
   Eigen::Vector3d coords2(0, 1, -2);
@@ -94,9 +96,10 @@ BOOST_AUTO_TEST_CASE(BoundingBoxCOG_3D)
   }
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Demonstration)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   for (int dim = 2; dim <= 3; dim++) {
     // Create mesh object
     std::string         meshName("MyMesh");
@@ -148,30 +151,13 @@ BOOST_AUTO_TEST_CASE(Demonstration)
     Edge &e2 = mesh.createEdge(v2, v0);
 
     BOOST_TEST(mesh.hasEdges());
+    BOOST_TEST(mesh.edges().size() == 3);
     BOOST_TEST(!mesh.hasTriangles());
 
-    // Validate mesh edges state
-    index = 0;
-    for (Edge &edge : mesh.edges()) {
-      if (index == 0) {
-        BOOST_TEST(edge.getID() == e0.getID());
-      } else if (index == 1) {
-        BOOST_TEST(edge.getID() == e1.getID());
-      } else if (index == 2) {
-        BOOST_TEST(edge.getID() == e2.getID());
-      } else {
-        BOOST_TEST(false);
-      }
-      index++;
-    }
-
-    Triangle *t = nullptr;
     if (dim == 3) {
       // Create triangle
-      t = &mesh.createTriangle(e0, e1, e2);
+      mesh.createTriangle(e0, e1, e2);
 
-      // Validate mesh triangle
-      BOOST_TEST((*mesh.triangles().begin()).getID() == t->getID());
       BOOST_TEST(mesh.hasTriangles());
     } else {
       BOOST_TEST(!mesh.hasTriangles());
@@ -209,15 +195,16 @@ BOOST_AUTO_TEST_CASE(Demonstration)
   }
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(MeshEquality)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   int                   dim = 3;
   Mesh                  mesh1("Mesh1", dim, testing::nextMeshID());
   Mesh                  mesh2("Mesh2", dim, testing::nextMeshID());
   std::array<Mesh *, 2> meshes = {&mesh1, &mesh2};
   for (auto ptr : meshes) {
-    auto &          mesh = *ptr;
+    auto           &mesh = *ptr;
     Eigen::VectorXd coords0(dim);
     Eigen::VectorXd coords1(dim);
     Eigen::VectorXd coords2(dim);
@@ -230,9 +217,9 @@ BOOST_AUTO_TEST_CASE(MeshEquality)
     Vertex &v1 = mesh.createVertex(coords1);
     Vertex &v2 = mesh.createVertex(coords2);
     Vertex &v3 = mesh.createVertex(coords3);
-    Edge &  e0 = mesh.createEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
-    Edge &  e1 = mesh.createEdge(v1, v2); // LINESTRING (1 0 0, 0 0 1)
-    Edge &  e2 = mesh.createEdge(v2, v0); // LINESTRING (0 0 1, 0 0 0)
+    Edge   &e0 = mesh.createEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
+    Edge   &e1 = mesh.createEdge(v1, v2); // LINESTRING (1 0 0, 0 0 1)
+    Edge   &e2 = mesh.createEdge(v2, v0); // LINESTRING (0 0 1, 0 0 0)
     mesh.createEdge(v1, v3);              // LINESTRING (1 0 0, 1 0 1)
     mesh.createEdge(v3, v2);              // LINESTRING (1 0 1, 0 0 1)
     mesh.createTriangle(e0, e1, e2);
@@ -240,17 +227,18 @@ BOOST_AUTO_TEST_CASE(MeshEquality)
   BOOST_TEST(mesh1 == mesh2);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(MeshWKTPrint)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh    mesh("WKTMesh", 3, testing::nextMeshID());
   Vertex &v0 = mesh.createVertex(Eigen::Vector3d(0., 0., 0.));
   Vertex &v1 = mesh.createVertex(Eigen::Vector3d(1., 0., 0.));
   Vertex &v2 = mesh.createVertex(Eigen::Vector3d(0., 0., 1.));
   Vertex &v3 = mesh.createVertex(Eigen::Vector3d(1., 0., 1.));
-  Edge &  e0 = mesh.createEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
-  Edge &  e1 = mesh.createEdge(v1, v2); // LINESTRING (1 0 0, 0 0 1)
-  Edge &  e2 = mesh.createEdge(v2, v0); // LINESTRING (0 0 1, 0 0 0)
+  Edge   &e0 = mesh.createEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
+  Edge   &e1 = mesh.createEdge(v1, v2); // LINESTRING (1 0 0, 0 0 1)
+  Edge   &e2 = mesh.createEdge(v2, v0); // LINESTRING (0 0 1, 0 0 0)
   mesh.createEdge(v1, v3);              // LINESTRING (1 0 0, 1 0 1)
   mesh.createEdge(v3, v2);              // LINESTRING (1 0 1, 0 0 1)
   mesh.createTriangle(e0, e1, e2);
@@ -260,53 +248,24 @@ BOOST_AUTO_TEST_CASE(MeshWKTPrint)
       "Mesh \"WKTMesh\", dimensionality = 3:\n"
       "GEOMETRYCOLLECTION(\n"
       "POINT (0 0 0), POINT (1 0 0), POINT (0 0 1), POINT (1 0 1),\n"
-      "LINESTRING (0 0 0, 1 0 0), LINESTRING (1 0 0, 0 0 1), LINESTRING (0 0 1, 0 0 0), LINESTRING (1 0 0, 1 0 1), LINESTRING (1 0 1, 0 0 1),\n"
+      "LINESTRING (0 0 0, 1 0 0), LINESTRING (1 0 0, 0 0 1), LINESTRING (0 0 0, 0 0 1), LINESTRING (1 0 0, 1 0 1), LINESTRING (0 0 1, 1 0 1),\n"
       "POLYGON ((0 0 0, 1 0 0, 0 0 1, 0 0 0))\n"
       ")");
   BOOST_TEST(reference == sstream.str());
 }
 
-BOOST_AUTO_TEST_CASE(CreateUniqueEdge)
-{
-  PRECICE_TEST(1_rank);
-  int             dim = 3;
-  Mesh            mesh1("Mesh1", dim, testing::nextMeshID());
-  auto &          mesh = mesh1;
-  Eigen::VectorXd coords0(dim);
-  Eigen::VectorXd coords1(dim);
-  Eigen::VectorXd coords2(dim);
-  coords0 << 0.0, 0.0, 0.0;
-  coords1 << 1.0, 0.0, 0.0;
-  coords2 << 0.0, 0.0, 1.0;
-  Vertex &v0 = mesh.createVertex(coords0);
-  Vertex &v1 = mesh.createVertex(coords1);
-  Vertex &v2 = mesh.createVertex(coords2);
-
-  Edge &e01a = mesh.createEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
-  mesh.createEdge(v0, v1);              // LINESTRING (0 0 0, 1 0 0)
-  BOOST_TEST(mesh.edges().size() == 2);
-
-  Edge &e01c = mesh.createUniqueEdge(v0, v1); // LINESTRING (0 0 0, 1 0 0)
-  BOOST_TEST(mesh.edges().size() == 2);
-  BOOST_TEST(e01a == e01c);
-
-  mesh.createUniqueEdge(v1, v2); // LINESTRING (0 0 0, 1 0 0)
-  BOOST_TEST(mesh.edges().size() == 3);
-  mesh.createUniqueEdge(v1, v2); // LINESTRING (0 0 0, 1 0 0)
-  BOOST_TEST(mesh.edges().size() == 3);
-}
-
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(ResizeDataGrow)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   precice::mesh::Mesh mesh("MyMesh", 3, testing::nextMeshID());
-  const auto &        values = mesh.createData("Data", 1, 0_dataID)->values();
+  const auto         &values = mesh.createData("Data", 1, 0_dataID)->values();
 
   // Create mesh
   mesh.createVertex(Vector3d(0.0, 0.0, 0.0));
   mesh.createVertex(Vector3d(1.0, 0.0, 1.0));
 
-  BOOST_TEST(mesh.vertices().size() == 2);
+  BOOST_TEST(mesh.nVertices() == 2);
   mesh.allocateDataValues();
   BOOST_TEST(values.size() == 2);
 
@@ -314,16 +273,17 @@ BOOST_AUTO_TEST_CASE(ResizeDataGrow)
   mesh.createVertex(Vector3d(2.0, 0.0, 2.0));
   mesh.createVertex(Vector3d(2.0, 0.0, 2.1));
 
-  BOOST_TEST(mesh.vertices().size() == 5);
+  BOOST_TEST(mesh.nVertices() == 5);
   mesh.allocateDataValues();
   BOOST_TEST(values.size() == 5);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(ResizeDataShrink)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   precice::mesh::Mesh mesh("MyMesh", 3, testing::nextMeshID());
-  const auto &        values = mesh.createData("Data", 1, 0_dataID)->values();
+  const auto         &values = mesh.createData("Data", 1, 0_dataID)->values();
 
   // Create mesh
   mesh.createVertex(Vector3d(0.0, 0.0, 0.0));
@@ -331,7 +291,7 @@ BOOST_AUTO_TEST_CASE(ResizeDataShrink)
   mesh.createVertex(Vector3d(1.0, 1.0, 1.0));
   mesh.createVertex(Vector3d(2.0, 2.0, 2.0));
 
-  BOOST_TEST(mesh.vertices().size() == 4);
+  BOOST_TEST(mesh.nVertices() == 4);
   mesh.allocateDataValues();
   BOOST_TEST(values.size() == 4);
 
@@ -339,16 +299,17 @@ BOOST_AUTO_TEST_CASE(ResizeDataShrink)
   mesh.createVertex(Vector3d(0.0, 0.0, 0.0));
   mesh.createVertex(Vector3d(1.0, 0.0, 1.0));
 
-  BOOST_TEST(mesh.vertices().size() == 2);
+  BOOST_TEST(mesh.nVertices() == 2);
   mesh.allocateDataValues();
   BOOST_TEST(values.size() == 2);
 }
 
 BOOST_AUTO_TEST_SUITE(Utils)
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(AsChain)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh mesh("Mesh1", 3, testing::nextMeshID());
   mesh.createData("Data", 1, 0_dataID);
 
@@ -365,7 +326,7 @@ BOOST_AUTO_TEST_CASE(AsChain)
   Vertex &v2 = mesh.createVertex(coords2);
   Vertex &v3 = mesh.createVertex(coords3);
 
-  BOOST_TEST(mesh.vertices().size() == 4);
+  BOOST_TEST(mesh.nVertices() == 4);
 
   Edge &e0 = mesh.createEdge(v0, v1);
   Edge &e1 = mesh.createEdge(v3, v2);
@@ -389,9 +350,10 @@ BOOST_AUTO_TEST_CASE(AsChain)
   BOOST_TEST(chain.vertices.at(3) == &v0);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(ShareVertex)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh mesh("Mesh1", 3, testing::nextMeshID());
   mesh.createData("Data", 1, 0_dataID);
 
@@ -407,7 +369,7 @@ BOOST_AUTO_TEST_CASE(ShareVertex)
   Vertex *v1 = &mesh.createVertex(coords1);
   Vertex *v2 = &mesh.createVertex(coords2);
   Vertex *v3 = &mesh.createVertex(coords3);
-  BOOST_REQUIRE(mesh.vertices().size() == 4);
+  BOOST_REQUIRE(mesh.nVertices() == 4);
 
   Edge &e0 = mesh.createEdge(*v0, *v1);
   Edge &e1 = mesh.createEdge(*v1, *v2);
@@ -429,9 +391,10 @@ BOOST_AUTO_TEST_CASE(ShareVertex)
   BOOST_TEST(sharedVertex(e0, e2) == sharedVertex(e2, e0));
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(EdgeLength)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
 
   Eigen::Vector3d coords0;
   Eigen::Vector3d coords1;
@@ -439,13 +402,14 @@ BOOST_AUTO_TEST_CASE(EdgeLength)
   coords1 << 0.0, 1.0, 0.0;
   Vertex v0{coords0, 0};
   Vertex v1{coords1, 1};
-  Edge   e(v0, v1, 0);
+  Edge   e(v0, v1);
   BOOST_TEST(edgeLength(e) == std::sqrt(2));
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(VertexPtrsFor)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh mesh("Mesh1", 3, testing::nextMeshID());
   mesh.createData("Data", 1, 0_dataID);
 
@@ -461,7 +425,7 @@ BOOST_AUTO_TEST_CASE(VertexPtrsFor)
   Vertex &v1 = mesh.createVertex(coords1);
   Vertex &v2 = mesh.createVertex(coords2);
   mesh.createVertex(coords3);
-  BOOST_TEST(mesh.vertices().size() == 4);
+  BOOST_TEST(mesh.nVertices() == 4);
 
   std::array<int, 3>      ids{v0.getID(), v2.getID(), v1.getID()};
   std::array<Vertex *, 3> expected{&v0, &v2, &v1};
@@ -471,9 +435,10 @@ BOOST_AUTO_TEST_CASE(VertexPtrsFor)
   BOOST_TEST(result == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(CoordsForIDs)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh mesh("Mesh1", 3, testing::nextMeshID());
   mesh.createData("Data", 1, 0_dataID);
 
@@ -489,7 +454,7 @@ BOOST_AUTO_TEST_CASE(CoordsForIDs)
   Vertex &v1 = mesh.createVertex(coords1);
   Vertex &v2 = mesh.createVertex(coords2);
   mesh.createVertex(coords3);
-  BOOST_TEST(mesh.vertices().size() == 4);
+  BOOST_TEST(mesh.nVertices() == 4);
 
   std::array<int, 3>             ids{v0.getID(), v2.getID(), v1.getID()};
   std::array<Eigen::VectorXd, 3> expected{coords0, coords2, coords1};
@@ -499,9 +464,10 @@ BOOST_AUTO_TEST_CASE(CoordsForIDs)
   BOOST_TEST(result == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(CoordsForPtrs)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   Mesh mesh("Mesh1", 3, testing::nextMeshID());
   mesh.createData("Data", 1, 0_dataID);
 
@@ -517,7 +483,7 @@ BOOST_AUTO_TEST_CASE(CoordsForPtrs)
   Vertex &v1 = mesh.createVertex(coords1);
   Vertex &v2 = mesh.createVertex(coords2);
   mesh.createVertex(coords3);
-  BOOST_TEST(mesh.vertices().size() == 4);
+  BOOST_TEST(mesh.nVertices() == 4);
 
   std::array<Vertex *, 3>        ptrs{&v0, &v2, &v1};
   std::array<Eigen::VectorXd, 3> expected{coords0, coords2, coords1};
@@ -527,9 +493,10 @@ BOOST_AUTO_TEST_CASE(CoordsForPtrs)
   BOOST_TEST(result == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Integrate2DScalarData)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 2, testing::nextMeshID());
   mesh->createData("Data", 1, 0_dataID);
 
@@ -548,15 +515,16 @@ BOOST_AUTO_TEST_CASE(Integrate2DScalarData)
   mesh->data(0)->values()(2) = 5.0;
   mesh->data(0)->values()(3) = 7.0;
 
-  auto   result   = mesh::integrate(mesh, mesh->data(0));
+  auto   result   = mesh::integrateSurface(mesh, mesh->data(0)->values());
   double expected = 17.0;
   BOOST_REQUIRE(result.size() == 1);
   BOOST_TEST(result(0) == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Integrate2DVectorData)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 2, testing::nextMeshID());
   mesh->createData("Data", 2, 0_dataID);
 
@@ -579,16 +547,17 @@ BOOST_AUTO_TEST_CASE(Integrate2DVectorData)
   mesh->data(0)->values()(6) = 7.0;
   mesh->data(0)->values()(7) = 8.0;
 
-  auto            result = mesh::integrate(mesh, mesh->data(0));
+  auto            result = mesh::integrateSurface(mesh, mesh->data(0)->values());
   Eigen::Vector2d expected(17.0, 20.5);
   BOOST_REQUIRE(result.size() == 2);
   BOOST_TEST(result(0) == expected(0));
   BOOST_TEST(result(1) == expected(1));
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Integrate3DScalarData)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 3, testing::nextMeshID());
   mesh->createData("Data", 1, 0_dataID);
 
@@ -612,15 +581,16 @@ BOOST_AUTO_TEST_CASE(Integrate3DScalarData)
   mesh->data(0)->values()(2) = 5.0;
   mesh->data(0)->values()(3) = 7.0;
 
-  auto   result   = mesh::integrate(mesh, mesh->data(0));
+  auto   result   = mesh::integrateSurface(mesh, mesh->data(0)->values());
   double expected = 70.0;
   BOOST_REQUIRE(result.size() == 1);
   BOOST_TEST(result(0) == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Integrate3DVectorData)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 3, testing::nextMeshID());
   mesh->createData("Data", 2, 0_dataID);
 
@@ -648,7 +618,7 @@ BOOST_AUTO_TEST_CASE(Integrate3DVectorData)
   mesh->data(0)->values()(6) = 7.0;
   mesh->data(0)->values()(7) = 8.0;
 
-  auto            result = mesh::integrate(mesh, mesh->data(0));
+  auto            result = mesh::integrateSurface(mesh, mesh->data(0)->values());
   Eigen::Vector2d expected(70.0, 88.0);
   BOOST_REQUIRE(result.size() == 2);
   BOOST_TEST(result(0) == expected(0));
@@ -666,9 +636,10 @@ struct UnitSquareFixture {
   Eigen::Vector2d x3{0.0, 1.0};
 };
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_FIXTURE_TEST_CASE(Integrate2DScalarDataVolume, UnitSquareFixture)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 2, testing::nextMeshID());
   mesh->createData("Data", 1, 0_dataID);
 
@@ -698,15 +669,16 @@ BOOST_FIXTURE_TEST_CASE(Integrate2DScalarDataVolume, UnitSquareFixture)
   mesh->data(0)->values()(2) = 7.0;
   mesh->data(0)->values()(3) = 5.0;
 
-  auto   result   = mesh::integrateVolume(mesh, mesh->data(0));
+  auto   result   = mesh::integrateVolume(mesh, mesh->data(0)->values());
   double expected = 4.0;
   BOOST_REQUIRE(result.size() == 1);
   BOOST_TEST(result(0) == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_FIXTURE_TEST_CASE(Integrate2DVectorDataVolume, UnitSquareFixture)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 2, testing::nextMeshID());
   mesh->createData("Data", 2, 0_dataID);
 
@@ -741,7 +713,7 @@ BOOST_FIXTURE_TEST_CASE(Integrate2DVectorDataVolume, UnitSquareFixture)
   mesh->data(0)->values()(5) = 8.0;
   mesh->data(0)->values()(7) = 6.0;
 
-  auto            result = mesh::integrateVolume(mesh, mesh->data(0));
+  auto            result = mesh::integrateVolume(mesh, mesh->data(0)->values());
   Eigen::Vector2d expected(4.0, 5.0);
   BOOST_REQUIRE(result.size() == 2);
   BOOST_TEST(result(0) == expected(0));
@@ -755,9 +727,10 @@ struct OneTetraFixture {
   Eigen::Vector3d x4{0.0, 0.0, 1.0};
 };
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_FIXTURE_TEST_CASE(Integrate3DScalarDataVolume, OneTetraFixture)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 3, testing::nextMeshID());
   mesh->createData("Data", 1, 0_dataID);
 
@@ -779,15 +752,16 @@ BOOST_FIXTURE_TEST_CASE(Integrate3DScalarDataVolume, OneTetraFixture)
   mesh->data(0)->values()(2) = 5.0;
   mesh->data(0)->values()(3) = 7.0;
 
-  auto   result   = mesh::integrateVolume(mesh, mesh->data(0));
+  auto   result   = mesh::integrateVolume(mesh, mesh->data(0)->values());
   double expected = 4.0 / 6;
   BOOST_REQUIRE(result.size() == 1);
   BOOST_TEST(result(0) == expected);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_FIXTURE_TEST_CASE(Integrate3DVectorDataVolume, OneTetraFixture)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh mesh = std::make_shared<Mesh>("Mesh1", 3, testing::nextMeshID());
   mesh->createData("Data", 3, 0_dataID);
 
@@ -819,7 +793,7 @@ BOOST_FIXTURE_TEST_CASE(Integrate3DVectorDataVolume, OneTetraFixture)
   mesh->data(0)->values()(2 * 3 + 2) = 0.0;
   mesh->data(0)->values()(3 * 3 + 2) = 0.0;
 
-  auto            result = mesh::integrateVolume(mesh, mesh->data(0));
+  auto            result = mesh::integrateVolume(mesh, mesh->data(0)->values());
   Eigen::Vector3d expected(4.0 / 6, 1.0 / 6, 1.0 / 24);
   BOOST_REQUIRE(result.size() == 3);
   BOOST_TEST(result(0) == expected(0));
@@ -829,9 +803,10 @@ BOOST_FIXTURE_TEST_CASE(Integrate3DVectorDataVolume, OneTetraFixture)
 
 BOOST_AUTO_TEST_SUITE_END() // VolumeIntegrals
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(AddMesh)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   PtrMesh globalMesh = std::make_shared<Mesh>("Mesh1", 3, testing::nextMeshID());
   PtrMesh subMesh    = std::make_shared<Mesh>("Mesh2", 3, testing::nextMeshID());
 
@@ -861,11 +836,292 @@ BOOST_AUTO_TEST_CASE(AddMesh)
   subMesh->createTetrahedron(v15, v12, v13, v14);
 
   globalMesh->addMesh(*subMesh);
-  BOOST_TEST(globalMesh->vertices().size() == 9);
+  BOOST_TEST(globalMesh->nVertices() == 9);
   BOOST_TEST(globalMesh->edges().size() == 3);
   BOOST_TEST(globalMesh->triangles().size() == 3);
   BOOST_TEST(globalMesh->tetrahedra().size() == 3);
 }
 
+BOOST_AUTO_TEST_SUITE(PreProcess);
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(DuplicateEdges)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(3.0, 0.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(3.0, 4.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.0, 8.0, 0.0));
+
+  mesh.createEdge(v1, v2);
+  mesh.createEdge(v2, v3);
+  mesh.createEdge(v3, v4);
+
+  // add single duplicate
+  mesh.createEdge(v3, v4);
+
+  // add 1000 duplicates
+  for (int i = 0; i < 1000; ++i) {
+    mesh.createEdge(v2, v3);
+  };
+  BOOST_TEST(mesh.edges().size() == 1004);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 3);
+  BOOST_TEST(mesh.triangles().empty());
+  BOOST_TEST(mesh.tetrahedra().empty());
+
+  std::vector<Edge> expectedEdges{{v1, v2}, {v2, v3}, {v3, v4}};
+  for (auto &e : expectedEdges) {
+    auto cnt = std::count(mesh.edges().begin(), mesh.edges().end(), e);
+    BOOST_TEST(cnt == 1);
+  }
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTriangle)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+
+  mesh.createTriangle(v1, v2, v3);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 3);
+  BOOST_TEST(mesh.triangles().size() == 1);
+  BOOST_TEST(mesh.tetrahedra().size() == 0);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTriangulatedQuad)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(1.0, 0.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createTriangle(v2, v3, v4);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 5);
+  BOOST_TEST(mesh.triangles().size() == 2);
+  BOOST_TEST(mesh.tetrahedra().size() == 0);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTetrahedron)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, 1.0));
+
+  mesh.createTetrahedron(v1, v2, v3, v4);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 6);
+  BOOST_TEST(mesh.triangles().size() == 4);
+  BOOST_TEST(mesh.tetrahedra().size() == 1);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(TouchingTetrahedra)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, 1.0));
+  auto &v5 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, -1.0));
+
+  mesh.createTetrahedron(v1, v2, v3, v4);
+  mesh.createTetrahedron(v1, v2, v3, v5);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 9);
+  BOOST_TEST(mesh.triangles().size() == 7);
+  BOOST_TEST(mesh.tetrahedra().size() == 2);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTriangle2ImplicitEdges)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createEdge(v2, v3);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 3);
+  BOOST_TEST(mesh.triangles().size() == 1);
+  BOOST_TEST(mesh.tetrahedra().size() == 0);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTriangle1ImplicitEdges)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createEdge(v1, v2);
+  mesh.createEdge(v2, v3);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 3);
+  BOOST_TEST(mesh.triangles().size() == 1);
+  BOOST_TEST(mesh.tetrahedra().size() == 0);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTetrahedron3ImplicitTriangles)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, 1.0));
+
+  mesh.createTetrahedron(v1, v2, v3, v4);
+  mesh.createTriangle(v1, v2, v3);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 6);
+  BOOST_TEST(mesh.triangles().size() == 4);
+  BOOST_TEST(mesh.tetrahedra().size() == 1);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(SingleTetrahedronNoImplicitTriangles)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, 1.0));
+
+  mesh.createTetrahedron(v1, v2, v3, v4);
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createTriangle(v1, v2, v4);
+  mesh.createTriangle(v3, v4, v1);
+  mesh.createTriangle(v3, v4, v2);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 6);
+  BOOST_TEST(mesh.triangles().size() == 4);
+  BOOST_TEST(mesh.tetrahedra().size() == 1);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(Mixed)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(0.0, 1.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(1.0, 1.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.3, 0.3, 1.0));
+
+  mesh.createTetrahedron(v1, v2, v3, v4);
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createEdge(v3, v4);
+
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 6);
+  BOOST_TEST(mesh.triangles().size() == 4);
+  BOOST_TEST(mesh.tetrahedra().size() == 1);
+}
+
+PRECICE_TEST_SETUP(1_rank)
+BOOST_AUTO_TEST_CASE(Complex)
+{
+  PRECICE_TEST();
+  Mesh mesh{"Mesh1", 3, 0};
+
+  auto &v1 = mesh.createVertex(Eigen::Vector3d(0.0, 0.0, 0.0));
+  auto &v2 = mesh.createVertex(Eigen::Vector3d(3.0, 0.0, 0.0));
+  auto &v3 = mesh.createVertex(Eigen::Vector3d(3.0, 4.0, 0.0));
+  auto &v4 = mesh.createVertex(Eigen::Vector3d(0.0, 8.0, 0.0));
+
+  mesh.createEdge(v1, v2);
+  mesh.createEdge(v2, v3);
+  mesh.createEdge(v3, v4);
+
+  // add single duplicate
+  mesh.createEdge(v3, v4);
+
+  // add 1000 duplicates
+  for (int i = 0; i < 1000; ++i) {
+    mesh.createEdge(v2, v3);
+  };
+  BOOST_TEST(mesh.edges().size() == 1004);
+
+  mesh.createTriangle(v1, v2, v3);
+  mesh.createTriangle(v2, v3, v4);
+  for (int i = 0; i < 1000; ++i) {
+    mesh.createTriangle(v2, v3, v4);
+  };
+  BOOST_TEST(mesh.triangles().size() == 1002);
+
+  // creates edges 1-3 and 2-4
+  mesh.preprocess();
+
+  BOOST_TEST(mesh.edges().size() == 5);
+  BOOST_TEST(mesh.triangles().size() == 2);
+
+  std::vector<Edge> expectedEdges{{v1, v2}, {v1, v3}, {v2, v3}, {v3, v4}, {v2, v4}};
+  for (auto &e : expectedEdges) {
+    auto cnt = std::count(mesh.edges().begin(), mesh.edges().end(), e);
+    BOOST_TEST(cnt == 1);
+  }
+
+  std::vector<Triangle> expectedTriangles{{v1, v2, v3}, {v2, v3, v4}};
+  for (auto &t : expectedTriangles) {
+    auto cnt = std::count(mesh.triangles().begin(), mesh.triangles().end(), t);
+    BOOST_TEST(cnt == 1);
+  }
+}
+
+BOOST_AUTO_TEST_SUITE_END();
 BOOST_AUTO_TEST_SUITE_END() // Mesh
 BOOST_AUTO_TEST_SUITE_END() // Mesh
