@@ -11,9 +11,10 @@ using namespace precice;
 BOOST_AUTO_TEST_SUITE(MeshTests)
 BOOST_AUTO_TEST_SUITE(VertexTests)
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(Vertices)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   mesh::Vertex vertex(Eigen::Vector3d::Constant(1.0), 0);
 
   Eigen::Vector3d coords = vertex.getCoords();
@@ -23,9 +24,10 @@ BOOST_AUTO_TEST_CASE(Vertices)
   BOOST_TEST(id == 0);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(VertexEquality)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   using namespace mesh;
   using namespace Eigen;
   Vertex v1(Vector3d::Constant(4.0), 0);
@@ -36,9 +38,10 @@ BOOST_AUTO_TEST_CASE(VertexEquality)
   BOOST_TEST(v2 != v3);
 }
 
+PRECICE_TEST_SETUP(1_rank)
 BOOST_AUTO_TEST_CASE(VertexWKTPrint)
 {
-  PRECICE_TEST(1_rank);
+  PRECICE_TEST();
   using namespace mesh;
   Vertex            v1(Eigen::Vector2d(1., 2.), 0);
   std::stringstream v1stream;
