@@ -8,7 +8,6 @@
 
 #include "action/Action.hpp"
 #include "action/config/ActionConfiguration.hpp"
-#include "com/MPIDirectCommunication.hpp"
 #include "com/SharedPointer.hpp"
 #include "com/config/CommunicationConfiguration.hpp"
 #include "io/ExportCSV.hpp"
@@ -35,6 +34,12 @@
 #include "xml/ConfigParser.hpp"
 #include "xml/XMLAttribute.hpp"
 
+#ifdef PRECICE_NO_MPI
+#include "com/SocketCommunication.hpp"
+#else
+#include "com/MPIDirectCommunication.hpp"
+#endif
+
 namespace precice::config {
 
 ParticipantConfiguration::ParticipantConfiguration(
@@ -274,7 +279,7 @@ void ParticipantConfiguration::xmlTagCallback(
     // Start with defining the mesh
     mesh::PtrMesh mesh = _meshConfig->getMesh(name);
     PRECICE_CHECK(mesh,
-                  R"(Participant "{}" attempts to provide an unknown mesh "{}". <mesh name="{}"> needs to be defined first.)",
+                  R"(Participant "{}" attempts to receive an unknown mesh "{}". <mesh name="{}"> needs to be defined first.)",
                   _participants.back()->getName(), name, name);
 
     // Then check the attributes
@@ -325,6 +330,11 @@ void ParticipantConfiguration::xmlTagCallback(
     config.isScalingOn = tag.getBooleanAttributeValue(ATTR_SCALE_WITH_CONN);
     _watchIntegralConfigs.push_back(config);
   } else if (tag.getNamespace() == TAG_INTRA_COMM) {
+    if (auto participant = _participants.back()->getName();
+        context.size == 1 && participant == context.name) {
+      PRECICE_INFO("Ignoring user-defined intra-comm for participant {} as it is running in serial.", participant);
+      return;
+    }
     com::CommunicationConfiguration comConfig;
     utils::IntraComm::getCommunication() = comConfig.createCommunication(tag);
     _isIntraCommDefined                  = true;
@@ -341,13 +351,18 @@ void ParticipantConfiguration::xmlEndTagCallback(
   }
 }
 
+std::size_t ParticipantConfiguration::nParticipants() const
+{
+  return _participants.size();
+}
+
 const std::vector<impl::PtrParticipant> &
 ParticipantConfiguration::getParticipants() const
 {
   return _participants;
 }
 
-const impl::PtrParticipant ParticipantConfiguration::getParticipant(const std::string &participantName) const
+const impl::PtrParticipant ParticipantConfiguration::getParticipant(std::string_view participantName) const
 {
   auto participant = std::find_if(_participants.begin(), _participants.end(), [&participantName](const auto &p) { return p->getName() == participantName; });
   PRECICE_ASSERT(participant != _participants.end(), "Did not find participant \"{}\"", participantName);
@@ -655,7 +670,7 @@ void ParticipantConfiguration::finishParticipantConfiguration(
         if (context.size > 1) {
           // Only display the warning message if this participant configuration is the current one.
           if (context.name == participant->getName()) {
-            PRECICE_ERROR("You attempted to use the legacy VTK exporter with the parallel participant {}, which isn't supported."
+            PRECICE_ERROR("You attempted to use the legacy VTK exporter with the parallel participant {}, which isn't supported. "
                           "Migrate to another exporter, such as the VTU exporter by specifying \"<export:vtu ... />\"  instead of \"<export:vtk ... />\".",
                           participant->getName());
           }
@@ -754,12 +769,15 @@ void ParticipantConfiguration::finishParticipantConfiguration(
   // create default primary communication if needed
   if (context.size > 1 && not _isIntraCommDefined && participant->getName() == context.name) {
 #ifdef PRECICE_NO_MPI
-    PRECICE_ERROR("Implicit intra-participant communications for parallel participants are only available if preCICE was built with MPI. "
-                  "Either explicitly define an intra-participant communication for each parallel participant or rebuild preCICE with \"PRECICE_MPICommunication=ON\".");
+    auto lo = utils::networking::loopbackInterfaceName();
+    PRECICE_INFO("Implicit intra-participant communications for parallel participants using preCICE without MPI defaults to a sockets intracomm using the default loopback device {}. "
+                 "Define your own <intra-comm:sockets ... /> to modify these defaults.",
+                 lo);
+    utils::IntraComm::getCommunication() = std::make_shared<com::SocketCommunication>(0, false, lo, ".");
 #else
     utils::IntraComm::getCommunication() = std::make_shared<com::MPIDirectCommunication>();
-    participant->setUsePrimaryRank(true);
 #endif
+    participant->setUsePrimaryRank(true);
   }
   _isIntraCommDefined = false; // to not mess up with previous participant
 }
