@@ -7,20 +7,17 @@
 #include "logging/Logger.hpp"
 #include "mesh/SharedPointer.hpp"
 
-namespace precice {
-namespace mesh {
+namespace precice::mesh {
 class Vertex;
 }
-} // namespace precice
 
-namespace precice {
-namespace impl {
+namespace precice::impl {
 
 /**
  * @brief Track and output transient integral data on a mesh
- * 
+ *
  * Calculation of integral depends on connectivity information of mesh and
- * given scaling option. If scale with area option is true, vertex data is 
+ * given scaling option. If scale with area option is true, vertex data is
  * weighted by area and summed up, otherwise, vertex data is directly summed
  * up.
  */
@@ -55,10 +52,9 @@ private:
 
   bool _isScalingOn;
 
-  Eigen::VectorXd calculateIntegral(const mesh::PtrData &data) const;
+  Eigen::VectorXd calculateIntegral(double time, const mesh::PtrData &data) const;
 
   double calculateSurfaceArea() const;
 };
 
-} // namespace impl
-} // namespace precice
+} // namespace precice::impl
