@@ -3,8 +3,7 @@
 #include <boost/container/flat_set.hpp>
 #include <functional>
 
-namespace precice {
-namespace utils {
+namespace precice::utils {
 
 /// Manages a set of unique IDs.
 class ManageUniqueIDs {
@@ -13,11 +12,11 @@ public:
   int getFreeID();
 
   /**
-    * @brief Inserts an ID which has to be unique.
-    *
-    * The inserted ID has to be different to all other IDs inserted and obtained
-    * from getFreeID().
-    */
+   * @brief Inserts an ID which has to be unique.
+   *
+   * The inserted ID has to be different to all other IDs inserted and obtained
+   * from getFreeID().
+   */
   bool insertID(int id);
 
   /// Resets all retrieved and inserted IDs.
@@ -31,5 +30,4 @@ private:
   int _lowerLimit = 0;
 };
 
-} // namespace utils
-} // namespace precice
+} // namespace precice::utils
