@@ -18,62 +18,24 @@
 #include "petscis.h"
 #include "petscksp.h"
 #include "petscsystypes.h"
+#include "petscvec.h"
 #include "petscviewertypes.h"
 #include "utils/Parallel.hpp"
+#include "utils/assertion.hpp"
 
 #endif // not PRECICE_NO_PETSC
 
-namespace precice {
-namespace utils {
+namespace precice::utils {
+
+precice::logging::Logger precice::utils::Petsc::_log("utils::Petsc");
 
 #ifndef PRECICE_NO_PETSC
-
-namespace {
-
-using new_signature = PetscErrorCode(PetscOptions, const char[], const char[]);
-using old_signature = PetscErrorCode(const char[], const char[]);
-
-/**
- * @brief Fix for compatibility with PETSc < 3.7. 
- * 
- * This enables to call PetscOptionsSetValue with proper number of arguments.
- * This instantiates only the template, that specifies correct function signature, whilst 
- * the other one is discarded ( https://en.cppreference.com/w/cpp/language/sfinae )
- */
-template <typename curr_signature = decltype(PetscOptionsSetValue)>
-PetscErrorCode PetscOptionsSetValueWrapper(const char name[], const char value[],
-                                           typename std::enable_if<std::is_same<curr_signature, new_signature>::value, curr_signature>::type PetscOptionsSetValueImpl =
-                                               PetscOptionsSetValue)
-{
-  return PetscOptionsSetValueImpl(nullptr, name, value);
-}
-
-/**
- * @brief Fix for compatibility with PETSc < 3.7. 
- * 
- * This enables to call PetscOptionsSetValue with proper number of arguments.
- * This instantiates only the template, that specifies correct function signature, whilst 
- * the other one is discarded ( https://en.cppreference.com/w/cpp/language/sfinae )
- */
-template <typename curr_signature = decltype(PetscOptionsSetValue)>
-PetscErrorCode PetscOptionsSetValueWrapper(const char name[], const char value[],
-                                           typename std::enable_if<std::is_same<curr_signature, old_signature>::value, curr_signature>::type PetscOptionsSetValueImpl =
-                                               PetscOptionsSetValue)
-{
-  return PetscOptionsSetValueImpl(name, value);
-}
-
-} // namespace
-#endif
-
-logging::Logger Petsc::_log("utils::Petsc");
+precice::logging::Logger precice::utils::petsc::Vector::_log("utils::Petsc::Vector");
+#endif // not PRECICE_NO_PETSC
 
 bool Petsc::weInitialized = false;
 
-void Petsc::initialize(
-    int *                  argc,
-    char ***               argv,
-    Parallel::Communicator comm)
+void Petsc::initialize(Parallel::Communicator comm)
 {
   PRECICE_TRACE();
 #ifndef PRECICE_NO_PETSC
@@ -81,11 +43,14 @@ void Petsc::initialize(
   PetscInitialized(&petscIsInitialized);
   if (not petscIsInitialized) {
     PETSC_COMM_WORLD = comm;
+    // Disable the default signal handler
+    PetscOptionsSetValue(nullptr, "-no_signal_handler", nullptr);
     PetscErrorCode ierr;
-    ierr = PetscInitialize(argc, argv, "", nullptr);
+    int            argc = 0;
+    char         **argv = nullptr;
+    ierr                = PetscInitialize(&argc, &argv, "", nullptr);
     CHKERRV(ierr);
     weInitialized = true;
-    PetscPushErrorHandler(&PetscMPIAbortErrorHandler, nullptr);
   }
 #endif // not PRECICE_NO_PETSC
 }
@@ -93,16 +58,18 @@ void Petsc::initialize(
 void Petsc::finalize()
 {
 #ifndef PRECICE_NO_PETSC
+  if (!weInitialized) {
+    return;
+  }
   PetscBool petscIsInitialized;
   PetscInitialized(&petscIsInitialized);
-  if (petscIsInitialized and weInitialized) {
-    PetscOptionsSetValueWrapper("-options_left", "no");
+  if (petscIsInitialized) {
+    PetscOptionsSetValue(nullptr, "-options_left", "no");
     PetscFinalize();
   }
 #endif // not PRECICE_NO_PETSC
 }
-} // namespace utils
-} // namespace precice
+} // namespace precice::utils
 
 #ifndef PRECICE_NO_PETSC
 
@@ -111,9 +78,7 @@ void Petsc::finalize()
 #include "petscdraw.h"
 #include "petscviewer.h"
 
-namespace precice {
-namespace utils {
-namespace petsc {
+namespace precice::utils::petsc {
 
 struct Viewer {
   Viewer(const std::string &filename, VIEWERFORMAT format, MPI_Comm comm)
@@ -324,7 +289,7 @@ void Vector::setValue(PetscInt row, PetscScalar value)
 void Vector::arange(double start, double stop)
 {
   PetscErrorCode ierr = 0;
-  PetscScalar *  a;
+  PetscScalar   *a;
   PetscInt       range_start, range_end, size;
   VecGetSize(vector, &size);
   VecGetOwnershipRange(vector, &range_start, &range_end);
@@ -354,11 +319,47 @@ void Vector::fillWithRandoms()
   PetscRandomDestroy(&rctx);
 }
 
+Vector &Vector::copyFrom(precice::span<const double> source)
+{
+  // This is collective, so we can only skip if the global size is 0
+  if (getSize() == 0) {
+    return *this;
+  }
+  PRECICE_ASSERT(static_cast<PetscInt>(source.size()) == getLocalSize());
+  PetscScalar   *data;
+  PetscErrorCode ierr = 0;
+  ierr                = VecGetArray(vector, &data);
+  PRECICE_ASSERT(ierr == 0);
+  std::copy(source.begin(), source.end(), data);
+  ierr = VecRestoreArray(vector, &data);
+  PRECICE_ASSERT(ierr == 0);
+  return *this;
+}
+
+Vector &Vector::copyTo(precice::span<double> destination)
+{
+  // This is collective, so we can only skip if the global size is 0
+  if (getSize() == 0) {
+    return *this;
+  }
+  auto localSize = getLocalSize();
+  PRECICE_ASSERT(static_cast<PetscInt>(destination.size()) == localSize);
+  PetscScalar   *data;
+  PetscErrorCode ierr = 0;
+  ierr                = VecGetArray(vector, &data);
+  PRECICE_ASSERT(ierr == 0);
+  auto dataEnd = std::next(data, localSize);
+  std::copy(data, dataEnd, destination.begin());
+  ierr = VecRestoreArray(vector, &data);
+  PRECICE_ASSERT(ierr == 0);
+  return *this;
+}
+
 void Vector::sort()
 {
   PetscErrorCode ierr = 0;
   PetscInt       size;
-  PetscReal *    a;
+  PetscReal     *a;
   ierr = VecGetArray(vector, &a);
   CHKERRV(ierr);
   ierr = VecGetSize(vector, &size);
@@ -776,8 +777,6 @@ void destroy(AO *ao)
   }
 }
 
-} // namespace petsc
-} // namespace utils
-} // namespace precice
+} // namespace precice::utils::petsc
 
 #endif // PRECICE_NO_PETSC
