@@ -2,14 +2,15 @@
 
 #include "testing/Testing.hpp"
 
-#include <precice/SolverInterface.hpp>
+#include <precice/precice.hpp>
 #include <vector>
 
 BOOST_AUTO_TEST_SUITE(Integration)
 BOOST_AUTO_TEST_SUITE(Parallel)
+PRECICE_TEST_SETUP("ParallelSolver"_on(3_ranks), "SerialSolver"_on(1_rank))
 BOOST_AUTO_TEST_CASE(PrimaryRankSockets)
 {
-  PRECICE_TEST("ParallelSolver"_on(3_ranks), "SerialSolver"_on(1_rank));
+  PRECICE_TEST();
 
   std::string myMeshName;
   if (context.isNamed("ParallelSolver")) {
@@ -17,10 +18,10 @@ BOOST_AUTO_TEST_CASE(PrimaryRankSockets)
   } else {
     myMeshName = "SerialMesh";
   }
-  precice::SolverInterface interface(context.name, context.config(), context.rank, context.size);
-  int                      meshID      = interface.getMeshID(myMeshName);
-  double                   position[2] = {0, 0};
-  interface.setMeshVertex(meshID, position);
+  precice::Participant interface(context.name, context.config(), context.rank, context.size);
+  auto                 meshName    = myMeshName;
+  double               position[2] = {0, 0};
+  interface.setMeshVertex(meshName, position);
   interface.initialize();
   interface.advance(1.0);
   interface.finalize();
