@@ -2,7 +2,7 @@
 
 #include "testing/Testing.hpp"
 
-#include <precice/SolverInterface.hpp>
+#include <precice/precice.hpp>
 #include "helpers.hpp"
 
 using namespace precice;
@@ -13,10 +13,11 @@ BOOST_AUTO_TEST_SUITE(Serial)
 BOOST_AUTO_TEST_SUITE(MappingNearestNeighborGradient)
 
 // Unidirectional Nearest Neighbor Gradient Read Mapping
+PRECICE_TEST_SETUP("A"_on(1_rank), "B"_on(1_rank))
 BOOST_AUTO_TEST_CASE(GradientTestUnidirectionalReadVector)
 {
-  PRECICE_TEST("A"_on(1_rank), "B"_on(1_rank))
-  testVectorGradientFunctions(context, false);
+  PRECICE_TEST();
+  testVectorGradientFunctions(context);
 }
 
 BOOST_AUTO_TEST_SUITE_END()
