@@ -8,6 +8,6 @@ using namespace precice;
 using precice::testing::TestContext;
 
 /// tests the vector gradient API functions writeBlockVectorGradient data and writeVectorGradientData
-void testVectorGradientFunctions(const TestContext &context, const bool writeBlockWise);
+void testVectorGradientFunctions(const TestContext &context);
 
-#endif
\ No newline at end of file
+#endif
